@@ -157,6 +157,11 @@ void silc_channel_message(SilcClient client, SilcClientConnection conn,
 	!strstr(type, "text/vnd")) {
       /* It is something textual, display it */
       message = (const unsigned char *)data;
+    } else {
+      printformat_module("fe-common/silc", server, channel->channel_name,
+			 MSGLEVEL_CRAP, SILCTXT_MESSAGE_DATA,
+			 nick == NULL ? "[<unknown>]" : nick->nick, type);
+      message = NULL;
     }
   }
 
@@ -235,6 +240,12 @@ void silc_private_message(SilcClient client, SilcClientConnection conn,
 	!strstr(type, "text/vnd")) {
       /* It is something textual, display it */
       message = (const unsigned char *)data;
+    } else {
+      printformat_module("fe-common/silc", server, NULL,
+			 MSGLEVEL_CRAP, SILCTXT_MESSAGE_DATA,
+			 sender->nickname ? sender->nickname : "[<unknown>]",
+			 type);
+      message = NULL;
     }
   }
 
@@ -486,7 +497,7 @@ void silc_notify(SilcClient client, SilcClientConnection conn,
 
     tmp = silc_client_chmode(mode,
 			     channel->channel_key ? 
-			     channel->channel_key->cipher->name : "",
+			     silc_cipher_get_name(channel->channel_key) : "",
 			     channel->hmac ? 
 			     silc_hmac_get_name(channel->hmac) : "");
     
@@ -923,12 +934,13 @@ void silc_command(SilcClient client, SilcClientConnection conn,
   switch (command) {
 
   case SILC_COMMAND_INVITE:
-    printformat_module("fe-common/silc", server, NULL,
-		       MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITING,
-		       cmd_context->argv[2], 
-		       (cmd_context->argv[1][0] == '*' ?
-			(char *)conn->current_channel->channel_name :
-			(char *)cmd_context->argv[1]));
+    if (cmd_context->argc > 2)
+      printformat_module("fe-common/silc", server, NULL,
+			 MSGLEVEL_CRAP, SILCTXT_CHANNEL_INVITING,
+			 cmd_context->argv[2], 
+			 (cmd_context->argv[1][0] == '*' ?
+			  (char *)conn->current_channel->channel_name :
+			  (char *)cmd_context->argv[1]));
     break;
 
   case SILC_COMMAND_DETACH:
@@ -1314,7 +1326,8 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 
       mode = silc_client_chmode(modei, 
 				channel_entry->channel_key ? 
-				channel_entry->channel_key->cipher->name : "",
+				silc_cipher_get_name(channel_entry->
+						     channel_key) : "",
 				channel_entry->hmac ? 
 				silc_hmac_get_name(channel_entry->hmac) : "");
       g_free_not_null(chanrec->mode);
@@ -1352,6 +1365,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
       char *topic, *name;
       int usercount;
       char users[20];
+      char tmp[256], *cp, *dm = NULL;
       
       if (!success)
 	return;
@@ -1360,6 +1374,20 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
       name = va_arg(vp, char *);
       topic = va_arg(vp, char *);
       usercount = va_arg(vp, int);
+
+      if (topic && !silc_term_utf8() &&
+	  silc_utf8_valid(topic, strlen(topic))) {
+	memset(tmp, 0, sizeof(tmp));
+	cp = tmp;
+	if (strlen(topic) > sizeof(tmp) - 1) {
+	  dm = silc_calloc(strlen(topic) + 1, sizeof(*dm));
+	  cp = dm;
+	}
+
+	silc_utf8_decode(topic, strlen(topic), SILC_STRING_LANGUAGE,
+			 cp, strlen(topic));
+	topic = cp;
+      }
       
       if (status == SILC_STATUS_LIST_START ||
 	  status == SILC_STATUS_OK)
@@ -1373,6 +1401,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
       printformat_module("fe-common/silc", server, NULL,
 			 MSGLEVEL_CRAP, SILCTXT_LIST,
 			 name, users, topic ? topic : "");
+      silc_free(dm);
     }
     break;
     
@@ -1571,13 +1600,28 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
     {
       SilcChannelEntry channel;
       char *topic;
+      char tmp[256], *cp, *dm = NULL;
       
       if (!success)
 	return;
       
       channel = va_arg(vp, SilcChannelEntry);
       topic = va_arg(vp, char *);
-      
+
+      if (topic && !silc_term_utf8() &&
+	  silc_utf8_valid(topic, strlen(topic))) {
+	memset(tmp, 0, sizeof(tmp));
+	cp = tmp;
+	if (strlen(topic) > sizeof(tmp) - 1) {
+	  dm = silc_calloc(strlen(topic) + 1, sizeof(*dm));
+	  cp = dm;
+	}
+
+	silc_utf8_decode(topic, strlen(topic), SILC_STRING_LANGUAGE,
+			 cp, strlen(topic));
+	topic = cp;
+      }
+
       if (topic) {
 	chanrec = silc_channel_find_entry(server, channel);
 	if (chanrec) {
@@ -1593,6 +1637,7 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
 			   MSGLEVEL_CRAP, SILCTXT_CHANNEL_TOPIC_NOT_SET,
 			   channel->channel_name);
       }
+      silc_free(dm);
     }
     break;
 
@@ -2202,18 +2247,29 @@ void silc_ftp(SilcClient client, SilcClientConnection conn,
 {
   SILC_SERVER_REC *server;
   char portstr[12];
-  FtpSession ftp = silc_calloc(1, sizeof(*ftp));
+  FtpSession ftp = NULL;
 
   SILC_LOG_DEBUG(("Start"));
 
   server = conn->context;
 
-  ftp->client_entry = client_entry;
-  ftp->session_id = session_id;
-  ftp->send = FALSE;
-  ftp->conn = conn;
-  silc_dlist_add(server->ftp_sessions, ftp);
-  server->current_session = ftp;
+  silc_dlist_start(server->ftp_sessions);
+  while ((ftp = silc_dlist_get(server->ftp_sessions)) != SILC_LIST_END) {
+    if (ftp->client_entry == client_entry &&
+	ftp->session_id == session_id) {
+      server->current_session = ftp;
+      break;
+    }
+  }
+  if (ftp == SILC_LIST_END) {
+    ftp = silc_calloc(1, sizeof(*ftp));
+    ftp->client_entry = client_entry;
+    ftp->session_id = session_id;
+    ftp->send = FALSE;
+    ftp->conn = conn;
+    silc_dlist_add(server->ftp_sessions, ftp);
+    server->current_session = ftp;
+  }
 
   if (hostname) 
     snprintf(portstr, sizeof(portstr) - 1, "%d", port);
