@@ -54,24 +54,250 @@
 extern "C" {
 #endif
 
-/* Forward declarations */
-typedef struct SilcClientStruct *SilcClient;
-typedef struct SilcClientConnectionStruct *SilcClientConnection;
-typedef struct SilcClientPingStruct SilcClientPing;
-typedef struct SilcClientAwayStruct SilcClientAway;
-typedef struct SilcClientKeyAgreementStruct *SilcClientKeyAgreement;
-typedef struct SilcClientFtpSessionStruct *SilcClientFtpSession;
-typedef struct SilcClientEntryStruct *SilcClientEntry;
-typedef struct SilcChannelEntryStruct *SilcChannelEntry;
-typedef struct SilcServerEntryStruct *SilcServerEntry;
-typedef struct SilcClientCommandStruct *SilcClientCommand;
-typedef struct SilcClientCommandContextStruct *SilcClientCommandContext;
-typedef struct SilcClientCommandReplyContextStruct 
-                                           *SilcClientCommandReplyContext;
-typedef struct SilcChannelUserStruct *SilcChannelUser;
+#include "client.h"
 
 /* General definitions */
 
+/****s* silcclient/SilcClientAPI/SilcClient
+ *
+ * NAME
+ *
+ *    typedef struct SilcClientStruct { ... } *SilcClient
+ *
+ * DESCRIPTION
+ *
+ *    This is the actual SILC Client structure which represents one
+ *    SILC Client.  It is allocated with the silc_client_alloc function
+ *    and given as argument to all SILC Client Library functions.  It
+ *    is initialized with silc_client_init function, and freed with
+ *    silc_client_free function.
+ *
+ * SOURCE
+ */
+struct SilcClientStruct {
+  /*
+   * The following fields are set by application
+   */
+  char *nickname;               /* Nickname, MAY be set by application  */
+  char *username;               /* Username, MUST be set by application */
+  char *hostname;               /* hostname, MUST be set by application */
+  char *realname;               /* Real name, MUST be set be application */
+  
+  SilcPublicKey public_key;     /* Public key of user, set by application */
+  SilcPrivateKey private_key;   /* Private key of user, set by application */
+  SilcPKCS pkcs;                /* PKCS allocated by application */
+
+  /*
+   * The following fields are set by the library
+   */
+
+  /* Scheduler, set by library.  Application may use this pointer. */
+  SilcSchedule schedule;
+  
+  /* Random Number Generator. Application should use this as its primary
+     random number generator. */
+  SilcRng rng;
+  
+  /* Application specific user data pointer. Client library does not
+     touch this.  This the context sent as argument to silc_client_alloc.
+     Application can use it freely. */
+  void *application;
+     
+  /* Generic hash context for application usage */
+  SilcHash md5hash;
+  SilcHash sha1hash;
+
+  /* Internal data for client library. Application cannot access this
+     data at all. */
+  SilcClientInternal internal;
+};  
+/***/
+
+/****s* silcclient/SilcClientAPI/SilcClientConnection
+ *
+ * NAME
+ *
+ *    typedef struct SilcClientConnectionStruct { ... }
+ *                      *SilcClientConnection
+ *
+ * DESCRIPTION
+ *
+ *    This structure represents a connection.  When connection is created
+ *    to server this is context is returned to the application in the
+ *    "connected" client operation.  It includes all the important
+ *    data for the session, such as nickname, local and remote IDs, and
+ *    other information.
+ *
+ * SOURCE
+ */
+struct SilcClientConnectionStruct {
+  /*
+   * Local data
+   */
+  char *nickname;		  /* Current nickname */
+  SilcClientEntry local_entry;	  /* Own Client Entry */
+  SilcClientID *local_id;	  /* Current Client ID */
+  unsigned char *local_id_data;	  /* Current Client ID decoded */
+  SilcUInt32 local_id_data_len;
+
+  /*
+   * Remote data
+   */
+  char *remote_host;		  /* Remote host name */
+  int remote_port;		  /* Remote port */
+  SilcServerID *remote_id;	  /* Remote Server ID */
+  unsigned char *remote_id_data;  /* Remote Server ID decoded */
+  SilcUInt32 remote_id_data_len;
+
+  /*
+   * Common data
+   */
+
+  /* Current command identifier for a command that was sent last.
+     Application may get the value from this variable to find out the
+     command identifier for last command. */
+  SilcUInt16 cmd_ident;
+
+  /* User data context. Library does not touch this. Application may
+     freely set and use this pointer for its needs. */
+  void *context;
+
+  /* Pointer back to the SilcClient.  Application may use this. */
+  SilcClient client;
+
+  /* Current channel.  Application may use and set this pointer if needed. */
+  SilcChannelEntry current_channel;
+
+  /* Socket connection object for this connection.  Application may
+     use this if needed.  The sock->user_data is back pointer to this
+     structure. */
+  SilcSocketConnection sock;
+
+  /* Internal data for client library. Application cannot access this
+     data at all. */
+  SilcClientConnectionInternal internal;
+};
+/***/
+
+/****s* silcclient/SilcClientAPI/SilcClientEntry
+ *
+ * NAME
+ *
+ *    typedef struct SilcClientEntryStruct { ... } *SilcClientEntry
+ *
+ * DESCRIPTION
+ *
+ *    This structure represents a client or a user in the SILC network.
+ *    The local user has this structure also and it can be accessed from
+ *    SilcClientConnection structure.  All other users in the SILC network
+ *    that are accessed using the Client Library routines will have their
+ *    own SilcClientEntry structure.  For example, when finding users by
+ *    their nickname the Client Library returns this structure back to
+ *    the application.
+ *
+ * SOURCE
+ */
+struct SilcClientEntryStruct {
+  /* General information */
+  char *nickname;		/* nickname */
+  char *username;		/* username */
+  char *hostname;		/* hostname */
+  char *server;			/* SILC server name */
+  char *realname;		/* Realname (userinfo) */
+
+  /* Mode, ID and other information */
+  SilcUInt32 mode;		/* User mode in SILC */
+  SilcClientID *id;		/* The Client ID */
+  SilcDList attrs;		/* Requested Attributes (maybe NULL) */
+  unsigned char *fingerprint;	/* Fingerprint of client's public key */
+  SilcUInt32 fingerprint_len;	/* Length of the fingerprint */
+
+  /* Private message keys */
+  SilcCipher send_key;		/* Private message key for sending */
+  SilcCipher receive_key;	/* Private message key for receiving */
+  unsigned char *key;		/* Set only if appliation provided the
+				   key material. NULL if the library 
+				   generated the key. */
+  SilcUInt32 key_len;		/* Key length */
+  SilcClientKeyAgreement ke;	/* Current key agreement context or NULL */
+
+  /* SilcClientEntry status information */
+  SilcEntryStatus status;	/* Status mask */
+  SilcHashTable channels;	/* All channels client has joined */
+  SilcUInt16 resolve_cmd_ident;	/* Command identifier when resolving */
+  bool generated;		/* TRUE if library generated `key' */
+  bool valid;			/* FALSE if this entry is not valid */
+};
+/***/
+
+/****s* silcclient/SilcClientAPI/SilcChannelEntry
+ *
+ * NAME
+ *
+ *    typedef struct SilcChannelEntryStruct { ... } *SilcChannelEntry
+ *
+ * DESCRIPTION
+ *
+ *    This structure represents a channel in the SILC network.  All
+ *    channels that the client are aware of or have joined in will be
+ *    represented as SilcChannelEntry.  The structure includes information
+ *    about the channel.
+ *
+ * SOURCE
+ */
+struct SilcChannelEntryStruct {
+  /* General information */
+  char *channel_name;		             /* Channel name */
+  SilcChannelID *id;			     /* Channel ID */
+  SilcUInt32 mode;			     /* Channel mode */
+
+  /* All clients that has joined this channel */
+  SilcHashTable user_list;
+
+  /* Channel keys */
+  SilcCipher channel_key;                    /* The channel key */
+  unsigned char *key;			     /* Raw key data */
+  SilcUInt32 key_len;		             /* Raw key data length */
+  unsigned char iv[SILC_CIPHER_MAX_IV_SIZE]; /* Current IV */
+  SilcHmac hmac;			     /* Current HMAC */
+
+  /* Channel private keys */
+  SilcDList private_keys;		     /* List of private keys or NULL */
+  SilcChannelPrivateKey curr_key;	     /* Current private key */
+
+  /* SilcChannelEntry status information */
+  SilcCipher old_channel_key;
+  SilcHmac old_hmac;
+  SilcTask rekey_task;
+  SilcUInt16 resolve_cmd_ident;		     /* Command identifier when
+						resolving this entry */
+};
+/***/
+
+/****s* silcclient/SilcClientAPI/SilcServerEntry
+ *
+ * NAME
+ *
+ *    typedef struct SilcServerEntryStruct { ... } *SilcServerEntry
+ *
+ * DESCRIPTION
+ *
+ *    This structure represents a server in the SILC network.  All servers
+ *    that the client is aware of and have for example resolved with
+ *    SILC_COMMAND_INFO command have their on SilcServerEntry structure.
+ *
+ * SOURCE
+ */
+struct SilcServerEntryStruct {
+  /* General information */
+  char *server_name;			     /* Server name */
+  char *server_info;			     /* Server info */
+  SilcServerID *server_id;		     /* Server ID */
+  SilcUInt16 resolve_cmd_ident;		     /* Command identifier when
+					        resolving this entry */
+};
+/***/
+
 /****d* silcclient/SilcClientAPI/SilcKeyAgreementStatus
  *
  * NAME
@@ -155,7 +381,8 @@ typedef struct {
  *
  * NAME
  *
- *    typedef struct { ... } SilcChannelPrivateKey;
+ *    typedef struct SilcChannelPrivateKeyStruct { ... }
+ *                      *SilcChannelPrivateKey;
  *
  * DESCRIPTION
  *
@@ -164,13 +391,13 @@ typedef struct {
  *
  * SOURCE
  */
-typedef struct SilcChannelPrivateKeyStruct {
+struct SilcChannelPrivateKeyStruct {
   char *name;			      /* Application given name */
   SilcCipher cipher;		      /* The cipher and key */
   SilcHmac hmac;		      /* The HMAC and hmac key */
   unsigned char *key;		      /* The key data */
   SilcUInt32 key_len;		      /* The key length */
-} *SilcChannelPrivateKey;
+};
 /***/
 
 /****f* silcclient/SilcClientAPI/SilcAskPassphrase
@@ -378,15 +605,15 @@ typedef struct {
      The `status' indicated whether the connection were successful.  If it
      is error value the application must always call the function
      silc_client_close_connection. */
-  void (*connect)(SilcClient client, SilcClientConnection conn,
-		  SilcClientConnectionStatus status);
+  void (*connected)(SilcClient client, SilcClientConnection conn,
+		    SilcClientConnectionStatus status);
 
   /* Called to indicate that connection was disconnected to the server.
      The `status' may tell the reason of the disconnection, and if the
      `message' is non-NULL it may include the disconnection message
      received from server. */
-  void (*disconnect)(SilcClient client, SilcClientConnection conn,
-		     SilcStatus status, const char *message);
+  void (*disconnected)(SilcClient client, SilcClientConnection conn,
+		       SilcStatus status, const char *message);
 
   /* Find authentication method and authentication data by hostname and
      port. The hostname may be IP address as well. When the authentication
@@ -605,7 +832,7 @@ typedef struct {
 SilcClient silc_client_alloc(SilcClientOperations *ops, 
 			     SilcClientParams *params,
 			     void *application,
-			     const char *silc_version);
+			     const char *version_string);
 
 /****f* silcclient/SilcClientAPI/silc_client_free
  *
@@ -647,6 +874,7 @@ int silc_client_init(SilcClient client);
  *
  *    Runs the client. This starts the scheduler from the utility library.
  *    When this functions returns the execution of the appliation is over.
+ *    The client must be initialized before calling this.
  *
  ***/
 void silc_client_run(SilcClient client);
@@ -1301,92 +1529,47 @@ SilcChannelUser silc_client_on_channel(SilcChannelEntry channel,
 
 /* Command management (command.c) */
 
-/****f* silcclient/SilcClientAPI/silc_client_command_alloc
- *
- * SYNOPSIS
- *
- *    SilcClientCommandContext silc_client_command_alloc(void);
- *
- * DESCRIPTION
- *
- *    Allocate Command Context. The context is defined in `command.h' file.
- *    The context is used by the library commands and applications should use
- *    it as well. However, application may choose to use some own context
- *    for its local commands. All library commands, however, must use this
- *    context. 
- *
- ***/
-SilcClientCommandContext silc_client_command_alloc(void);
-
-/****f* silcclient/SilcClientAPI/silc_client_command_free
- *
- * SYNOPSIS
- *
- *    void silc_client_command_free(SilcClientCommandContext ctx);
- *
- * DESCRIPTION
- *
- *    Free command context and its internals.  If the contex was duplicated
- *    with silc_client_command_dup this may not actually free the data, 
- *    instead it will decrease the reference counter of the context.  The
- *    context will be freed when the reference counter hits zero.
- *
- ***/
-void silc_client_command_free(SilcClientCommandContext ctx);
-
-/****f* silcclient/SilcClientAPI/silc_client_command_dup
- *
- * SYNOPSIS
- *
- *    SilcClientCommandContext 
- *    silc_client_command_dup(SilcClientCommandContext ctx);
- *
- * DESCRIPTION
- *
- *    Duplicate Command Context by adding reference counter. The context won't
- *    be free'd untill it hits zero. 
- *
- ***/
-SilcClientCommandContext silc_client_command_dup(SilcClientCommandContext ctx);
-
-/****f* silcclient/SilcClientAPI/silc_client_command_find
- *
- * SYNOPSIS
- *
- *    SilcClientCommand silc_client_command_find(SilcClient client,
- *                                               const char *name);
- *
- * DESCRIPTION
- *
- *    Finds and returns a pointer to the command list. Return NULL if the
- *    command is not found. See the `command.[ch]' for the command list. 
- *    Command names are not case-sensitive.
- *
- ***/
-SilcClientCommand silc_client_command_find(SilcClient client,
-					   const char *name);
-
 /****f* silcclient/SilcClientAPI/silc_client_command_call
  *
  * SYNOPSIS
  *
- *    void silc_client_command_call(SilcClientCommand command);
+ *    void silc_client_command_call(SilcClient client,
+ *                                  SilcClientConnection conn,
+ *                                  const char *command_line, ...);
  *
  * DESCRIPTION
  *
- *    Calls the command (executes it).  Application can call this after
- *    it has allocated the SilcClientCommandContext with the function
- *    silc_client_command_alloc and found the command from the client
- *    library by calling silc_client_command_find.  This will execute
- *    the command.
+ *    Calls and executes the command indicated by the `command_name'.
+ *    The `command_line' is a string which includes the command's name and
+ *    its arguments separated with whitespaces (' ').  If `command_line'
+ *    is non-NULL then all variable arguments are ignored by default.
  *
- *    Application can call the command function directly too if it
- *    wishes to do so.  See the command.h for details of the
- *    SilcClientCommand structure.
+ *    If `command_line' is NULL, then the variable arguments define the
+ *    command's name and its arguments.  The first variable argument must
+ *    be the command name.  The variable argument list must be terminated
+ *    with NULL.
+ *
+ *    Returns FALSE if the command is not known and TRUE after command.
+ *    execution.  The "command" client operation is called when the
+ *    command is executed to indicate whether the command executed
+ *    successfully or not.
+ *
+ *    The "command_reply" client operation will be called when reply is
+ *    received from the server to the command.  Application may also use
+ *    the silc_client_command_pending to attach to the command reply.
+ *    The command identifier for silc_client_command_pending function after
+ *    this function call is conn->cmd_ident, which application may use.
+ *
+ * EXAMPLE
+ *
+ *    silc_client_command_call(client, conn, NULL, "PING", "silc.silcnet.org",
+ *                             NULL);
+ *    silc_client_command_call(client, conn, "PING silc.silcnet.org");
  *
  ***/
-void silc_client_command_call(SilcClientCommand command,
-			      SilcClientCommandContext cmd);
+bool silc_client_command_call(SilcClient client,
+			      SilcClientConnection conn,
+			      const char *command_line, ...);
 
 /****f* silcclient/SilcClientAPI/silc_client_command_send
  *
@@ -1404,6 +1587,16 @@ void silc_client_command_call(SilcClientCommand command,
  *    to perform the commands by itself, it can do so and send the data
  *    directly to the server using this function.  If application is using
  *    the silc_client_command_call, this function is usually not used.
+ *    Note that this overriders the Client Librarys commands and sends
+ *    the command packet directly to server.
+ *
+ *    The variable arguments are a pair of { type, data, data_length },
+ *    and the `argc' is the number of these pairs.
+ *
+ * EXAMPLE
+ *
+ *    silc_client_command_send(client, conn, SILC_COMMAND_WHOIS, 0, 1,
+ *                             1, nickname, strlen(nickname));
  *
  ***/
 void silc_client_command_send(SilcClient client, SilcClientConnection conn,
@@ -1422,15 +1615,40 @@ void silc_client_command_send(SilcClient client, SilcClientConnection conn,
  *
  * DESCRIPTION
  *
- *    Add new pending command to be executed when reply to a command has been
- *    received.  The `reply_cmd' is the command that will call the `callback'
- *    with `context' when reply has been received.  If `ident' is non-zero
- *    the `callback' will be executed when received reply with command 
- *    identifier `ident'. 
+ *    This function can be used to add pending command callback to be
+ *    called when an command reply is received to an earlier sent command.
+ *    The `reply_cmd' is the command that must be received in order for
+ *    the pending command callback indicated by `callback' to be called.
  *
- *    Note that the application is notified about the received command
+ *    The `ident' is a command identifier which was set for the earlier
+ *    sent command.  The command reply will include the same identifier
+ *    and pending command callback will be called when the reply is 
+ *    received with the same command identifier.  It is possible to
+ *    add multiple pending command callbacks for same command and for
+ *    same identifier.
+ *
+ *    Application may use this function to add its own command reply
+ *    handlers if it wishes not to use the standard `command_reply'
+ *    client operation.  However, note that the pending command callback
+ *    does not deliver parsed command reply, but application must parse
+ *    it itself.
+ *
+ *    Note also that the application is notified about the received command
  *    reply through the `command_reply' client operation before calling
- *    the `callback` pending command callback.
+ *    the `callback` pending command callback.  That is the normal 
+ *    command reply handling, and is called regardless whether pending
+ *    command callbacks are used or not.
+ *
+ *    Commands that application calls with silc_client_command_call
+ *    will use a command identifier from conn->cmd_ident variable.  After
+ *    calling the silc_client_command_call, the conn->cmd_ident includes
+ *    the command identifier that was used for the command sending.
+ *
+ * EXAMPLE
+ *
+ *    silc_client_command_call(client, conn, "PING silc.silcnet.org");
+ *    silc_client_command_pending(conn, SILC_COMMAND_PING, conn->cmd_ident,
+ *                                my_ping_handler, my_ping_context);
  *
  ***/
 void silc_client_command_pending(SilcClientConnection conn,
@@ -1929,7 +2147,6 @@ void silc_client_set_away_message(SilcClient client,
 				  SilcClientConnection conn,
 				  char *message);
 
-
 /****f* silcclient/SilcClientAPI/SilcConnectionAuthRequest
  *
  * SYNOPSIS
@@ -2078,6 +2295,7 @@ typedef void (*SilcClientFileMonitor)(SilcClient client,
  *                          void *monitor_context,
  *                          const char *local_ip,
  *                          SilcUInt32 local_port,
+ *                          bool do_not_bind,
  *                          SilcClientEntry client_entry,
  *                          const char *filepath);
  *                          SilcUInt32 *session_id);
@@ -2091,7 +2309,7 @@ typedef void (*SilcClientFileMonitor)(SilcClient client,
  *    transmission of the file.
  *
  *    This returns a file session ID for the file transmission to the
- *    `session_id' pointer..  It can be used to close the session (and
+ *    `session_id' pointer.  It can be used to close the session (and
  *    abort the file transmission) by calling the silc_client_file_close
  *    function.  The session ID is also returned in the `monitor' callback. 
  *
@@ -2099,9 +2317,9 @@ typedef void (*SilcClientFileMonitor)(SilcClient client,
  *    listener for key exchange protocol to that IP.  If `local_port' is
  *    non-zero that port is used.  If `local_ip' is NULL then this will
  *    automatically attempt to bind it to local IP address of the machine.
- *    If that fails then this does not bind to any address and port, and
- *    assume that the remote client will provide the listener for the
- *    key exchange protocol.
+ *    If `do_not_bind' is TRUE then the `local_ip' and `local_port' are
+ *    ignored and it is expected that the receiver will provide the
+ *    point of contact.  This is usefull if the sender is behind NAT.
  *
  *    If error will occur during the file transfer process the error
  *    status will be returned in the monitor callback.  In this case
@@ -2116,6 +2334,7 @@ silc_client_file_send(SilcClient client,
 		      void *monitor_context,
 		      const char *local_ip,
 		      SilcUInt32 local_port,
+		      bool do_not_bind,
 		      SilcClientEntry client_entry,
 		      const char *filepath,
 		      SilcUInt32 *session_id);
@@ -2281,7 +2500,34 @@ const SilcHashTable silc_client_attributes_get(SilcClient client,
  ***/
 SilcBuffer silc_client_attributes_request(SilcAttribute attribute, ...);
 
-#include "client.h"
+/* Low level packet sending functions */
+
+/****f* silcclient/SilcClientAPI/silc_client_send_packet
+ *
+ * SYNOPSIS
+ *
+ *     bool silc_client_send_packet(SilcClient client, 
+ *                                  SilcClientConnection conn,
+ *                                  SilcPacketType type, 
+ *                                  const unsigned char *data, 
+ *                                  SilcUInt32 data_len);
+ *
+ * DESCRIPTION
+ *
+ *    This routine can be used by application to send packets directly
+ *    to a connection indicated by `conn'.  Usually application does not
+ *    need this routine since the Client Library handles the packet
+ *    sending.  The `type' indicates the packet type.  If `data' is
+ *    NULL then empty packet is sent.  This returns FALSE if packet cannot 
+ *    be sent.
+ *
+ ***/
+bool silc_client_send_packet(SilcClient client, 
+			     SilcClientConnection conn,
+			     SilcPacketType type, 
+			     const unsigned char *data, 
+			     SilcUInt32 data_len);
+
 #include "command.h"
 #include "command_reply.h"
 #include "idlist.h"
