@@ -19,7 +19,7 @@
 /*
  * Created: Fri Jul 25 18:52:14 1997
  */
-/* $Id: silcpacket.c,v 1.55 2002/03/31 18:01:40 priikone Exp $ */
+/* $Id: silcpacket.c,v 1.58 2002/10/24 12:18:59 priikone Exp $ */
 
 #include "silcincludes.h"
 
@@ -101,8 +101,9 @@ void silc_packet_encrypt(SilcCipher cipher, SilcHmac hmac, SilcUInt32 sequence,
   /* Encrypt the data area of the packet. */
   if (cipher) {
     SILC_LOG_DEBUG(("Encrypting packet, cipher %s, len %d", 
-		    cipher->cipher->name, len));
-    silc_cipher_encrypt(cipher, buffer->data, buffer->data, len, cipher->iv);
+		    silc_cipher_get_name(cipher), len));
+    silc_cipher_encrypt(cipher, buffer->data, buffer->data, len,
+			silc_cipher_get_iv(cipher));
   }
 
   /* Pull the HMAC into the visible data area in the buffer */
@@ -326,14 +327,15 @@ bool silc_packet_receive_process(SilcSocketConnection sock,
     /* Decrypt first 16 bytes of the packet */
     if (!SILC_IS_INBUF_PENDING(sock) && cipher)
       silc_cipher_decrypt(cipher, sock->inbuf->data, sock->inbuf->data, 
-			  SILC_PACKET_MIN_HEADER_LEN, cipher->iv);
+			  SILC_PACKET_MIN_HEADER_LEN,
+			  silc_cipher_get_iv(cipher));
 
     /* Get packet lenght and full packet length with padding */
     SILC_PACKET_LENGTH(sock->inbuf, packetlen, paddedlen);
 
     /* Sanity checks */
     if (packetlen < SILC_PACKET_MIN_LEN) {
-      SILC_LOG_DEBUG(("Received invalid packet, dropped"));
+      SILC_LOG_ERROR(("Received invalid packet, dropped"));
       silc_buffer_clear(sock->inbuf);
       return FALSE;
     }
@@ -389,12 +391,16 @@ bool silc_packet_receive_process(SilcSocketConnection sock,
       if (silc_packet_decrypt(cipher, hmac, parse_ctx->packet->sequence, 
 			      parse_ctx->packet->buffer, 
 			      parse_ctx->normal) == -1) {
-	SILC_LOG_WARNING(("Packet decryption failed %s:%d [%s]", 
+	SILC_LOG_WARNING(("Packet decryption failed %s:%d [%s] [%s]", 
 			  sock->hostname, sock->port,
+			  silc_get_packet_name(parse_ctx->packet->type),
 			  (sock->type == SILC_SOCKET_TYPE_UNKNOWN ? "Unknown" :
 			   sock->type == SILC_SOCKET_TYPE_CLIENT ? "Client" :
 			   sock->type == SILC_SOCKET_TYPE_SERVER ? "Server" :
 			   "Router")));
+	silc_packet_context_free(parse_ctx->packet);
+	silc_free(parse_ctx);
+	return FALSE;
       }
 
     /* Pull the packet from inbuf thus we'll get the next one
@@ -463,7 +469,7 @@ static int silc_packet_decrypt_rest(SilcCipher cipher, SilcHmac hmac,
       if ((buffer->len - silc_hmac_len(hmac)) > SILC_PACKET_MIN_LEN) {
 	silc_buffer_push_tail(buffer, silc_hmac_len(hmac));
       } else {
-	SILC_LOG_DEBUG(("Bad MAC length in packet, packet dropped"));
+	SILC_LOG_ERROR(("Bad MAC length in packet, packet dropped"));
 	return FALSE;
       }
     }
@@ -473,7 +479,7 @@ static int silc_packet_decrypt_rest(SilcCipher cipher, SilcHmac hmac,
     /* Decrypt rest of the packet */
     silc_buffer_pull(buffer, SILC_PACKET_MIN_HEADER_LEN);
     silc_cipher_decrypt(cipher, buffer->data, buffer->data, buffer->len, 
-			cipher->iv);
+			silc_cipher_get_iv(cipher));
     silc_buffer_push(buffer, SILC_PACKET_MIN_HEADER_LEN);
 
     SILC_LOG_HEXDUMP(("Fully decrypted packet, len %d", buffer->len),
@@ -502,7 +508,7 @@ static int silc_packet_decrypt_rest_special(SilcCipher cipher,
       if ((buffer->len - silc_hmac_len(hmac)) > SILC_PACKET_MIN_LEN) {
 	silc_buffer_push_tail(buffer, silc_hmac_len(hmac));
       } else {
-	SILC_LOG_DEBUG(("Bad MAC length in packet, packet dropped"));
+	SILC_LOG_ERROR(("Bad MAC length in packet, packet dropped"));
 	return FALSE;
       }
     }
@@ -517,11 +523,12 @@ static int silc_packet_decrypt_rest_special(SilcCipher cipher,
 
     silc_buffer_pull(buffer, SILC_PACKET_MIN_HEADER_LEN);
     if (len > buffer->len) {
-      SILC_LOG_DEBUG(("Garbage in header of packet, bad packet length, "
+      SILC_LOG_ERROR(("Garbage in header of packet, bad packet length, "
 		      "packet dropped"));
       return FALSE;
     }
-    silc_cipher_decrypt(cipher, buffer->data, buffer->data, len, cipher->iv);
+    silc_cipher_decrypt(cipher, buffer->data, buffer->data, len,
+			silc_cipher_get_iv(cipher));
     silc_buffer_push(buffer, SILC_PACKET_MIN_HEADER_LEN);
     SILC_LOG_HEXDUMP(("packet, len %d", buffer->len), 
 		     buffer->data, buffer->len);
