@@ -50,9 +50,6 @@ typedef struct {
   SilcUInt32 retry_count;
   SilcUInt32 retry_timeout;
 
-  /* Back pointer to server */
-  SilcServer server;
-
   SilcServerConnectRouterCallback callback;
   void *callback_context;
 } *SilcServerConnection;
@@ -60,7 +57,7 @@ typedef struct {
 /* General definitions */
 
 /* SILC port */
-#define SILC_PORT 706;
+#define SILC_PORT 706
 
 /* Server and router. Used internally by the code. */
 #define SILC_SERVER 0
@@ -86,6 +83,10 @@ typedef struct {
 #define SILC_SERVER_MAX_CONNECTIONS    1000	 /* Max connections */
 #define SILC_SERVER_MAX_CONNECTIONS_SINGLE 1000  /* Max connections per host */
 #define SILC_SERVER_LOG_FLUSH_DELAY    300       /* Default log flush delay */
+#define SILC_SERVER_QOS_RATE_LIMIT     10        /* Default QoS rate limit */
+#define SILC_SERVER_QOS_BYTES_LIMIT    2048      /* Default QoS bytes limit */
+#define SILC_SERVER_QOS_LIMIT_SEC      0         /* Default QoS limit sec */
+#define SILC_SERVER_QOS_LIMIT_USEC     500000    /* Default QoS limit usec */
 
 /* Macros */
 
@@ -114,6 +115,11 @@ do {									\
   (sock->protocol && sock->protocol->protocol && 			\
    sock->protocol->protocol->type == SILC_PROTOCOL_SERVER_REKEY)
 
+/* Check whether backup resuming protocol is active */
+#define SILC_SERVER_IS_BACKUP(sock)					\
+  (sock->protocol && sock->protocol->protocol && 			\
+   sock->protocol->protocol->type == SILC_PROTOCOL_SERVER_BACKUP)
+
 /* Output an error message wether to stderr or LOG_ERROR if we are in the
    background. */
 #define SILC_SERVER_LOG_ERROR(fmt) silc_server_stderr(silc_format fmt)
@@ -236,10 +242,6 @@ SilcBuffer silc_server_get_client_channel_list(SilcServer server,
 					       bool get_private,
 					       bool get_secret,
 					       SilcBuffer *user_mode_list);
-SilcClientEntry silc_server_get_client_resolve(SilcServer server,
-					       SilcClientID *client_id,
-					       bool always_resolve,
-					       bool *resolved);
 void silc_server_stderr(char *message);
 
 #endif
